package com.menagerie.ophelia.view

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.modifier.modifierLocalConsumer
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringArrayResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.AnnotatedString
import androidx.compose.ui.text.SpanStyle
import androidx.compose.ui.text.buildAnnotatedString
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.unit.dp
import com.menagerie.ophelia.R

@Composable
fun OpheliaSpanStyle(): SpanStyle {
    val fontFamily = FontFamily(
        Font(R.font.calligraffitti)
    )

    return SpanStyle(
        letterSpacing = MaterialTheme.typography.bodyLarge.letterSpacing,
        fontFamily = fontFamily,
        fontSize = MaterialTheme.typography.bodyLarge.fontSize)
}

@Composable
fun OpheliaFace(
    modifier: Modifier
) {
    Image(
        modifier = modifier,
        painter = painterResource(id = R.drawable.ophelia_foreground),
        contentDescription = OpheliaSays()
    )
}

@Composable
fun OpheliaSaysArray(
    id: Int,
    separator: String = "\n\n"
): AnnotatedString {
    return buildAnnotatedString {
        pushStyle(OpheliaSpanStyle())
        append(stringArrayResource(id = id).joinToString(separator = separator))
        toAnnotatedString()
    }
}

@Composable
fun OpheliaSays(
    say: String
): AnnotatedString {
    return buildAnnotatedString {
        pushStyle(OpheliaSpanStyle())
        append(say)
        toAnnotatedString()
    }
}

@Composable
fun OpheliaSays(): String {
    return stringResource(id = R.string.ophelia_says)
}


@Composable
fun OpheliaWelcome(
    modifier: Modifier
) {
    Box(
        modifier = modifier
            .fillMaxSize()
            .padding(24.dp)
    ) {
        Text(
            text = OpheliaSaysArray(id = R.array.welcome_blurb)
        )
    }
}

@Composable
fun OpheliaWhatsYourName(
    modifier: Modifier
) {
    Box(
        modifier = modifier
            .fillMaxWidth()
            .padding(12.dp)
    ) {
        Text(text = OpheliaSaysArray(id = R.array.whats_your_name))
    }
}

@Composable
fun OpheliaPronounsIntroduction(
    modifier: Modifier,
    name: String
) {
    Box(
        modifier = modifier
            .fillMaxWidth()
            .padding(12.dp)
    ) {
        Column {
            val context = LocalContext.current
            Text(text = OpheliaSays(context.resources.getString(R.string.introduction_with_name, name)))
            Text(text = OpheliaSaysArray(id = R.array.introduction_to_pronouns))
        }
    }
}

@Composable
fun OpheliaTagsIntroduction(
    modifier: Modifier,
) {
    Box(
        modifier = modifier
            .fillMaxWidth()
            .padding(12.dp)
    ) {
        Column {
            Text(text = OpheliaSaysArray(R.array.introduction_to_tags))
        }
    }
}

@Composable
fun OpheliaNewUserFinalCheck(
    modifier: Modifier,
){
    Box(
        modifier = modifier
            .fillMaxWidth()
            .padding(12.dp)
    ) {
        Column {
            Text(text = OpheliaSays(stringResource(id = R.string.final_check_in)))
        }
    }
}

@Composable
fun OpheliaNewUserPolycule(
    modifier: Modifier,
) {
    Box(
        modifier = modifier
            .fillMaxWidth()
            .padding(12.dp)
    ) {
        Column {
            Text(text = OpheliaSaysArray(id = R.array.introduction_to_polycules))
        }
    }
}
