package com.menagerie.ophelia.view.biographies

import android.annotation.SuppressLint
import android.widget.Toast
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.CircleShape
import androidx.compose.foundation.text.KeyboardActions
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.FilterChip
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.focus.FocusDirection
import androidx.compose.ui.graphics.painter.Painter
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.LocalFocusManager
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewmodel.compose.viewModel
import com.menagerie.ophelia.R
import com.menagerie.ophelia.database.polycule.entity.Bio
import com.menagerie.ophelia.database.polycule.entity.viewmodel.BioDetailViewModel
import com.menagerie.ophelia.database.polycule.entity.viewmodel.BioListViewModel
import com.menagerie.ophelia.view.components.InputFieldState
import com.menagerie.ophelia.view.components.fab.FABComponent
import com.menagerie.ophelia.view.newUser.Tags

fun insertBioInDB(bio: Bio?, mBioListViewModel: BioListViewModel) {
    bio?.let {
        mBioListViewModel.upsert(it)
    }
}

@Composable
fun FinishBiography(
    bioId: Long,
    onGo: () -> Unit
) {
    val mBioDetailViewModel = BioDetailViewModel(bioId)
    val bio = mBioDetailViewModel.bio.observeAsState().value

    if (bio != null) {
        val inputViewModel = InputEditViewModel(bio)
        val ioBio: Bio by inputViewModel.bio.observeAsState(bio)
        AddEditBio(
            ioBio = ioBio,
            btnLabel = stringResource(id = R.string.finish),
            toast = stringResource(id = R.string.bio_finish_toast),
            onGo,
        )
    }
}

@Composable
fun EditBiography(
    bioId: Long,
    onGo: () -> Unit
) {
    val mBioDetailViewModel = BioDetailViewModel(bioId)
    val bio = mBioDetailViewModel.bio.observeAsState().value

    if (bio != null) {
        val inputViewModel = InputEditViewModel(bio)
        val ioBio: Bio by inputViewModel.bio.observeAsState(bio)
        AddEditBio(
            ioBio = ioBio,
            btnLabel = stringResource(id = R.string.edit_bio),
            toast = stringResource(id = R.string.bio_edit_toast),
            onGo,
        )
    }
}

@Composable
fun AddBiography() {

    val inputViewModel = InputAddViewModel()
    val ioBio: Bio by inputViewModel.bio.observeAsState(Bio())
    AddEditBio(
        ioBio = ioBio,
        btnLabel = stringResource(id = R.string.edit_bio),
        toast = stringResource(id = R.string.bio_edit_toast),
    ) {

    }
}

@SuppressLint("UnusedMaterial3ScaffoldPaddingParameter")
@Composable
fun AddEditBio(
    ioBio: Bio,
    btnLabel: String,
    toast: String,
    onGo: () -> Unit
) {

    val mBioListViewModel: BioListViewModel = viewModel(
        factory = BioListViewModel.BioListViewModelFactory()
    )
    val context = LocalContext.current

    var name by remember { mutableStateOf(ioBio.name) }
    var sp by remember { mutableStateOf(ioBio.subjective) }
    var op by remember { mutableStateOf(ioBio.objective) }
    var pp by remember { mutableStateOf(ioBio.possessive) }
    var rp by remember { mutableStateOf(ioBio.reflexive) }
    var tags by remember { mutableIntStateOf(ioBio.tags) }

    Scaffold(
        floatingActionButton = {
            FABComponent(text = btnLabel, onClick = {
                insertBioInDB(
                    Bio(
                        id = ioBio.id,
                        name = name,
                        subjective = sp,
                        objective = op,
                        possessive = pp,
                        reflexive = rp,
                        tags = tags,
                    ),
                    mBioListViewModel
                )
                onGo()
                Toast.makeText(context, toast, Toast.LENGTH_SHORT).show()
            })
        }
    )
    {
        Column(
            modifier = Modifier.padding(12.dp)
        ) {
            val modifier = Modifier
                .width(164.dp)
                .height(64.dp)
            val focusManager = LocalFocusManager.current
            InputFieldState(
                value = name,
                label = "name",
                modifier = modifier,
                keyboardActions = KeyboardActions(
                    onDone = { focusManager.moveFocus(FocusDirection.Next) }
                )
            ) { name = it }
            Box {
                Column {
                    Row {
                        InputFieldState(
                            value = sp,
                            label = "objective",
                            modifier = modifier,
                            keyboardActions = KeyboardActions(
                                onDone = { focusManager.moveFocus(FocusDirection.Next) }
                            )
                        ) { sp = it }
                        Text(
                            text = " / ",
                            style = MaterialTheme.typography.displayMedium
                        )
                        InputFieldState(
                            value = op,
                            label = "subjective",
                            modifier = modifier,
                            keyboardActions = KeyboardActions(
                                onDone = { focusManager.moveFocus(FocusDirection.Next) }
                            )
                        ) { op = it }
                    }
                    Row {
                        InputFieldState(
                            value = pp,
                            label = "possessive",
                            modifier = modifier,
                            keyboardActions = KeyboardActions(
                                onDone = { focusManager.moveFocus(FocusDirection.Next) }
                            )
                        ) { pp = it }
                        Text(
                            text = " / ",
                            style = MaterialTheme.typography.displayMedium
                        )
                        InputFieldState(
                            value = rp,
                            label = "reflexive",
                            modifier = modifier,
                            keyboardActions = KeyboardActions(
                                onDone = { focusManager.clearFocus() }
                            )
                        ) { rp = it }
                    }
                    tagsBox(
                        tags = tags,
                        modifier = Modifier
                            .padding(12.dp)
                            .size(64.dp)
                    ) {
                        tags = it
                    }
                }
            }
        }
    }
}

@Composable
fun tagsBox(
    tags: Int,
    modifier: Modifier,
    onValChange: (Int) -> Unit
) {

    Row {
        RememberableButton(
            tagList = tags,
            tag = Tags.TRANS,
            painter = painterResource(id = R.drawable.trans_tag),
            modifier = modifier,
            onValChange = onValChange)

        RememberableButton(
            tagList = tags,
            tag = Tags.FURRY,
            painter = painterResource(id = R.drawable.furry_tag),
            modifier = modifier,
            onValChange = onValChange)

        RememberableButton(
            tagList = tags,
            tag = Tags.ASEXUAL,
            painter = painterResource(id = R.drawable.asexual_tag),
            modifier = modifier,
            onValChange = onValChange)
    }


    Row {
        RememberableButton(
            tagList = tags,
            tag = Tags.BDSM,
            painter = painterResource(id = R.drawable.bdsm_tag),
            modifier = modifier,
            onValChange = onValChange)

        RememberableButton(
            tagList = tags,
            tag = Tags.MONO,
            painter = painterResource(id = R.drawable.mono_tag),
            modifier = modifier,
            onValChange = onValChange)

        RememberableButton(
            tagList = tags,
            tag = Tags.ALCOHOL,
            painter = painterResource(id = R.drawable.liqour_tag),
            modifier = modifier,
            onValChange = onValChange)
    }

    Row {
        RememberableButton(
            tagList = tags,
            tag = Tags.WEED,
            painter = painterResource(id = R.drawable.weed_tag),
            modifier = modifier,
            onValChange = onValChange)

        RememberableButton(
            tagList = tags,
            tag = Tags.THERIAN,
            painter = painterResource(id = R.drawable.therian_tag),
            modifier = modifier,
            onValChange = onValChange)

        RememberableButton(
            tagList = tags,
            tag = Tags.PLURAL,
            painter = painterResource(id = R.drawable.plural_tag),
            modifier = modifier,
            onValChange = onValChange)
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun RememberableButton(
    tagList: Int,
    tag: Tags,
    modifier: Modifier,
    painter: Painter,
    onValChange: (Int) -> Unit,
) {
    var selected = tagList and tag.value == tag.value

    Box {
        FilterChip(
            selected = selected,
            onClick = { onValChange(tagList xor tag.value) },
            modifier = modifier,
            shape = CircleShape,
            label = {
                Icon(
                    modifier = Modifier.padding(horizontal = 4.dp),
                    painter = painter,
                    contentDescription = ""
                )
            },
        )
    }
}

class InputEditViewModel(
    bio: Bio
) : ViewModel() {
    private val _bio: MutableLiveData<Bio> = MutableLiveData(bio)
    val bio: LiveData<Bio> = _bio
}

class InputAddViewModel() : ViewModel() {
    private val _bio: MutableLiveData<Bio> = MutableLiveData(Bio())
    val bio: LiveData<Bio> = _bio
}